@@ -1,131 +1,35 @@
 import abc
 import datetime
-import glob
 import json
 import os
 import pathlib
-import pickle
 import re
 import subprocess
 import tempfile
 from functools import cached_property
 
 from hotsos.core.config import HotSOSConfig
-from hotsos.core.host_helpers.common import HostHelpersBase
+from hotsos.core.host_helpers.common import (
+    CmdOutput,
+    get_ps_axo_flags_available,
+    HostHelpersBase,
+    reset_command,
+    run_post_exec_hooks,
+    run_pre_exec_hooks,
+    SourceRunner,
+)
+from hotsos.core.host_helpers.exceptions import (
+    catch_exceptions,
+    CLI_COMMON_EXCEPTIONS,
+    CLIExecError,
+    CommandNotFound,
+    SourceNotFound,
+)
 from hotsos.core.log import log
 
-CLI_COMMON_EXCEPTIONS = (OSError, subprocess.CalledProcessError,
-                         subprocess.TimeoutExpired,
-                         json.JSONDecodeError)
 
-
-class CLIExecError(Exception):
-
-    def __init__(self, return_value=None):
-        """
-        @param return_value: default return value that a command
-                             should return if execution fails.
-        """
-        self.return_value = return_value
-
-
-def catch_exceptions(*exc_types):
-    def catch_exceptions_inner1(f):
-        def catch_exceptions_inner2(*args, **kwargs):
-            try:
-                return f(*args, **kwargs)
-            except exc_types as exc:
-                msg = "{}: {}".format(type(exc), exc)
-                if isinstance(exc, subprocess.TimeoutExpired):
-                    log.info(msg)
-                else:
-                    log.debug(msg)
-
-                if isinstance(exc, json.JSONDecodeError):
-                    raise CLIExecError(return_value={}) from exc
-
-                raise CLIExecError(return_value=[]) from exc
-
-        return catch_exceptions_inner2
-
-    return catch_exceptions_inner1
-
-
-class SourceNotFound(Exception):
-    def __init__(self, path):
-        self.path = path
-
-    def __repr__(self):
-        return "source path '{}' not found".format(self.path)
-
-
-class CommandNotFound(Exception):
-    def __init__(self, cmd, msg):
-        self.msg = "command '{}' not found in catalog: '{}'".format(cmd, msg)
-
-    def __str__(self):
-        return self.msg
-
-
-class NullSource(object):
-    def __call__(self, *args, **kwargs):
-        return CmdOutput([])
-
-
-def run_pre_exec_hooks(f):
-    """ pre-exec hooks are run before running __call__ method.
-
-    These hooks are not expected to return anything and are used to manipulate
-    the instance variables used by the main __call__ method.
-    """
-    def run_pre_exec_hooks_inner(self, *args, **kwargs):
-        hook = self.hooks.get("pre-exec")
-        if hook:
-            # no return expected
-            hook(*args, **kwargs)
-
-        return f(self, *args, **kwargs)
-
-    return run_pre_exec_hooks_inner
-
-
-def run_post_exec_hooks(f):
-    """ post-exec hooks are run after running __call__ method and take its
-    output as input.
-    """
-    def run_post_exec_hooks_inner(self, *args, **kwargs):
-        out = f(self, *args, **kwargs)
-        hook = self.hooks.get("post-exec")
-        if hook:
-            out = hook(out, *args, **kwargs)
-
-        return out
-
-    return run_post_exec_hooks_inner
-
-
-def reset_command(f):
-    """
-    This should be run by all commands as their last action after all/any hooks
-    have run.
-    """
-    def reset_command_inner(self, *args, **kwargs):
-        out = f(self, *args, **kwargs)
-        self.reset()
-        return out
-
-    return reset_command_inner
-
-
-class CmdOutput(object):
-    def __init__(self, value, source=None):
-        self.value = value
-        self.source = source
-
-
-class CmdBase(object):
+class CmdBase():
     """ Base class for all command source types. """
-
     def __init__(self):
         self.hooks = {}
         self.reset()
@@ -141,7 +45,7 @@ class CmdBase(object):
 
     @classmethod
     def safe_readlines(cls, path):
-        with open(path, 'r', errors="surrogateescape") as fd:
+        with open(path, 'r', encoding='utf-8', errors="surrogateescape") as fd:
             return fd.readlines()
 
     def register_hook(self, name, f):
@@ -158,13 +62,14 @@ class CmdBase(object):
 
 
 class BinCmd(CmdBase):
+    """ Implements binary command execution. """
     TYPE = "BIN"
 
     def __init__(self, cmd, json_decode=False, singleline=False):
         """
         @param cmd: command in string format (not list)
         """
-        self.original_cmd = cmd
+        self.cmd = self.original_cmd = cmd
         self.original_cmd_extras = []
         self.original_json_decode = json_decode
         self.original_singleline = singleline
@@ -215,11 +120,17 @@ class BinCmd(CmdBase):
 
 
 class FileCmd(CmdBase):
+    """ Implements file-based command execution.
+
+    This is used e.g. with sosreports where the output of a command is saved
+    to disk.
+    """
     TYPE = "FILE"
 
     def __init__(self, path, json_decode=False,
                  singleline=False, decode_error_handling=None):
-        self.original_path = os.path.join(HotSOSConfig.data_root, path)
+        self.path = self.original_path = os.path.join(HotSOSConfig.data_root,
+                                                      path)
         self.original_json_decode = json_decode
         self.original_singleline = singleline
         self.original_decode_error_handling = decode_error_handling
@@ -251,7 +162,7 @@ class FileCmd(CmdBase):
         # NOTE: any post-exec hooks must be aware that their input will be
         # defined by the following.
         if self.json_decode:
-            with open(self.path) as fd:
+            with open(self.path, encoding='utf-8') as fd:
                 output = json.load(fd)
         else:
             output = []
@@ -294,8 +205,6 @@ class BinFileCmd(FileCmd):
     @run_post_exec_hooks
     @run_pre_exec_hooks
     def __call__(self, *args, **kwargs):
-        # TODO: find a better way to handle this because path may still need
-        # formatting.
         if not os.path.exists(self.original_path):
             raise SourceNotFound(self.original_path)
 
@@ -323,8 +232,8 @@ class BinFileCmd(FileCmd):
         return CmdOutput(output.splitlines(keepends=True))
 
 
-class JournalctlBase(object):
-
+class JournalctlBase():
+    """ Base class for journalctl command implementations. """
     @property
     def since_date(self):
         """
@@ -346,7 +255,7 @@ class JournalctlBase(object):
 
 
 class JournalctlBinCmd(BinCmd, JournalctlBase):
-
+    """ Implements binary journalctl command. """
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.register_hook("pre-exec", self.format_journalctl_cmd)
@@ -354,33 +263,33 @@ class JournalctlBinCmd(BinCmd, JournalctlBase):
     def format_journalctl_cmd(self, **kwargs):
         """ Add optional extras to journalctl command. """
         if kwargs.get("unit"):
-            self.cmd = "{} --unit {}".format(self.cmd, kwargs.get("unit"))
+            self.cmd = f"{self.cmd} --unit {kwargs.get('unit')}"
 
         if kwargs.get("date"):
-            self.cmd = "{} --since {}".format(self.cmd, kwargs.get("date"))
+            self.cmd = f"{self.cmd} --since {kwargs.get('date')}"
         else:
-            self.cmd = "{} --since {}".format(self.cmd, self.since_date)
+            self.cmd = f"{self.cmd} --since {self.since_date}"
 
 
 class JournalctlBinFileCmd(BinFileCmd, JournalctlBase):
-
+    """ Implements file-based journalctl command. """
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.register_hook("pre-exec", self.preformat_sos_journalctl)
 
     def preformat_sos_journalctl(self, **kwargs):
-        self.path = "journalctl -oshort-iso -D {}".format(self.path)
+        self.path = f"journalctl -oshort-iso -D {self.path}"
         if kwargs.get("unit"):
-            self.path = "{} --unit {}".format(self.path, kwargs.get("unit"))
+            self.path = f"{self.path} --unit {kwargs.get('unit')}"
 
         if kwargs.get("date"):
-            self.path = "{} --since {}".format(self.path, kwargs.get("date"))
+            self.path = f"{self.path} --since {kwargs.get('date')}"
         else:
-            self.path = "{} --since {}".format(self.path, self.since_date)
+            self.path = f"{self.path} --since {self.since_date}"
 
 
 class OVSAppCtlBinCmd(BinCmd):
-
+    """ Implements ovs-appctl binary command. """
     def __call__(self, *args, **kwargs):
         # Set defaults for optional args
         for key in ['flags', 'args']:
@@ -391,11 +300,11 @@ class OVSAppCtlBinCmd(BinCmd):
 
 
 class OVSAppCtlFileCmd(FileCmd):
-
+    """ Implements ovs-appctl file-based command. """
     def __call__(self, *args, **kwargs):
         for key in ['flags', 'args']:
             if key in kwargs:
-                kwargs[key] = '_{}'.format(kwargs[key])
+                kwargs[key] = f'_{kwargs[key]}'
             else:
                 kwargs[key] = ''
 
@@ -414,19 +323,24 @@ class OVSAppCtlFileCmd(FileCmd):
         return out
 
 
-class OVSOFCtlCmdBase(object):
-    OFPROTOCOL_VERSIONS = ['OpenFlow15', 'OpenFlow14', 'OpenFlow13',
-                           'OpenFlow12', 'OpenFlow11', 'OpenFlow10']
+OFPROTOCOL_VERSIONS = [
+    "OpenFlow15",
+    "OpenFlow14",
+    "OpenFlow13",
+    "OpenFlow12",
+    "OpenFlow11",
+    "OpenFlow10",
+]
 
 
-class OVSOFCtlBinCmd(OVSOFCtlCmdBase, BinCmd):
-
+class OVSOFCtlBinCmd(BinCmd):
+    """ Implementation of ovs-ofctl binary command. """
     def __call__(self, *args, **kwargs):
         """
         First try without specifying protocol version. If error is raised
         try with different versions until we get a result.
         """
-        self.cmd = "ovs-ofctl {}".format(self.cmd)
+        self.cmd = f"ovs-ofctl {self.cmd}"
         try:
             return super().__call__(*args, **kwargs)
         except CLIExecError:
@@ -437,11 +351,11 @@ class OVSOFCtlBinCmd(OVSOFCtlCmdBase, BinCmd):
         # catch_exceptions decorator and [] returned. We have no way of knowing
         # if that was the actual return or an exception was raised so we just
         # go ahead and retry with specific OF versions until we get a result.
-        for ver in self.OFPROTOCOL_VERSIONS:
+        for ver in OFPROTOCOL_VERSIONS:
             log.debug("%s: trying again with protocol version %s",
                       self.__class__.__name__, ver)
             self.reset()
-            self.cmd = "ovs-ofctl -O {} {}".format(ver, self.cmd)
+            self.cmd = f"ovs-ofctl -O {ver} {self.cmd}"
             try:
                 return super().__call__(*args, **kwargs)
             except CLIExecError:
@@ -451,19 +365,19 @@ class OVSOFCtlBinCmd(OVSOFCtlCmdBase, BinCmd):
         return CmdOutput([])
 
 
-class OVSOFCtlFileCmd(OVSOFCtlCmdBase, FileCmd):
-
+class OVSOFCtlFileCmd(FileCmd):
+    """ Implementation of ovs-ofctl file-based command. """
     def __call__(self, *args, **kwargs):
         """
         We do this in reverse order to bin command since it won't actually
         raise an error.
         """
-        for ver in self.OFPROTOCOL_VERSIONS:
+        for ver in OFPROTOCOL_VERSIONS:
             log.debug("%s: trying again with protocol version %s",
                       self.__class__.__name__, ver)
             self.reset()
             try:
-                kwargs['ofversion'] = '_-O_{}'.format(ver)
+                kwargs['ofversion'] = f'_-O_{ver}'
                 out = super().__call__(*args, **kwargs)
                 if out.value and 'version negotiation failed' in out.value[0]:
                     continue
@@ -479,9 +393,11 @@ class OVSOFCtlFileCmd(OVSOFCtlCmdBase, FileCmd):
             log.debug("%s: command with no protocol version failed",
                       self.__class__.__name__)
 
+        return None
+
 
 class DateBinCmd(BinCmd):
-
+    """ Implementation binary date command. """
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.register_hook("pre-exec", self.format_date_cmd)
@@ -493,14 +409,14 @@ class DateBinCmd(BinCmd):
         if not no_format and fmt is None:
             fmt = '+%s'
 
-        self.cmd = '{} --utc'.format(self.cmd)
+        self.cmd = f'{self.cmd} --utc'
         if fmt:
             # this can't get split() so add to the end of the command list
             self.original_cmd_extras = [fmt]
 
 
 class DateFileCmd(FileCmd):
-
+    """ Implementation of file-based date command. """
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.register_hook("post-exec", self.format_date)
@@ -532,8 +448,8 @@ class DateFileCmd(FileCmd):
             tz = 'UTC+8'
 
         # Include tz if available and then convert to utc
-        date = "{} {} {}".format(ret[1], tz, ret[3])
-        cmd = ["date", "--utc", "--date={}".format(date)]
+        date = f"{ret[1]} {tz} {ret[3]}"
+        cmd = ["date", "--utc", f"--date={date}"]
         if fmt:
             cmd.append(fmt)
 
@@ -567,7 +483,7 @@ class CephJSONFileCmd(FileCmd):
         if not os.path.exists(self.path):
             raise SourceNotFound(self.path)
 
-        with open(self.path) as f:
+        with open(self.path, encoding='utf-8') as f:
             lines = f.readlines()
 
         if self.first_line_filter:
@@ -595,111 +511,8 @@ class CephJSONFileCmd(FileCmd):
         return output
 
 
-class SourceRunner(object):
-
-    def __init__(self, cmdkey, sources, cache, output_file=None):
-        """
-        @param cmdkey: unique key identifying this command.
-        @param sources: list of command source implementations.
-        @param cache: CLICacheWrapper object.
-        """
-        self.cmdkey = cmdkey
-        self.sources = sources
-        self.cache = cache
-        self.output_file = output_file
-        # Command output can differ between CLIHelper and CLIHelperFile so we
-        # need to cache them separately.
-        if output_file:
-            self.cache_cmdkey = "{}.file".format(cmdkey)
-        else:
-            self.cache_cmdkey = cmdkey
-
-    def bsource(self, *args, **kwargs):
-        # binary sources only apply if data_root is system root
-        bin_out = None
-        for bsource in [s for s in self.sources if s.TYPE == "BIN"]:
-            cache = False
-            # NOTE: we currently only support caching commands with no
-            #       args.
-            if not any([args, kwargs]):
-                cache = True
-                out = self.cache.load(self.cache_cmdkey)
-                if out is not None:
-                    return out
-
-            try:
-                if self.output_file:
-                    # don't decode if we are going to be saving to a file
-                    kwargs['skip_json_decode'] = True
-
-                bin_out = bsource(*args, **kwargs)
-                if cache and bin_out is not None:
-                    try:
-                        self.cache.save(self.cache_cmdkey, bin_out)
-                    except pickle.PicklingError as exc:
-                        log.info("unable to cache command '%s' output: %s",
-                                 self.cmdkey, exc)
-
-                # if command executed but returned nothing that still counts
-                # as success.
-                break
-            except CLIExecError as exc:
-                bin_out = CmdOutput(exc.return_value)
-
-        return bin_out
-
-    def fsource(self, *args, **kwargs):
-        for fsource in [s for s in self.sources if s.TYPE == "FILE"]:
-            try:
-                skip_load_contents = False
-                if self.output_file:
-                    skip_load_contents = True
-
-                return fsource(*args, **kwargs,
-                               skip_load_contents=skip_load_contents)
-            except CLIExecError as exc:
-                return CmdOutput(exc.return_value)
-            except SourceNotFound:
-                pass
-
-    def _execute(self, *args, **kwargs):
-        # always try file sources first
-        ret = self.fsource(*args, **kwargs)
-        if ret is not None:
-            return ret
-
-        if HotSOSConfig.data_root != '/':
-            return NullSource()()
-
-        return self.bsource(*args, **kwargs)
-
-    def __call__(self, *args, **kwargs):
-        """
-        Execute the command using the appropriate source runner. These can be
-        binary or file-based depending on whether data root points to / or a
-        sosreport. File-based are attempted first.
-
-        A command can have more than one source implementation so we must
-        ensure they all have a chance to run.
-        """
-        out = self._execute(*args, **kwargs)
-        if self.output_file:
-            if out.source is not None:
-                return out.source
-
-            with open(self.output_file, 'w') as fd:
-                if isinstance(out.value, list):
-                    fd.write(''.join(out.value))
-                else:
-                    fd.write(out.value)
-
-                return self.output_file
-
-        return out.value
-
-
-class CLICacheWrapper(object):
-
+class CLICacheWrapper():
+    """ Wrapper for cli cache. """
     def __init__(self, cache_load_f, cache_save_f):
         self.load_f = cache_load_f
         self.save_f = cache_save_f
@@ -712,7 +525,7 @@ class CLICacheWrapper(object):
 
 
 class CLIHelperBase(HostHelpersBase):
-
+    """ Base class for clihelper implementations. """
     def __init__(self):
         self._command_catalog = None
         super().__init__()
@@ -734,8 +547,8 @@ class CLIHelperBase(HostHelpersBase):
     def cache_load(self, key):
         return self.cache.get(key)
 
-    def cache_save(self, key, output):
-        return self.cache.set(key, output)
+    def cache_save(self, key, value):
+        return self.cache.set(key, value)
 
     @property
     def command_catalog(self):
@@ -1117,18 +930,4 @@ class CLIHelperFile(CLIHelperBase):
         except KeyError as exc:
             raise CommandNotFound(cmdname, exc) from exc
 
-
-def get_ps_axo_flags_available():
-    path = os.path.join(HotSOSConfig.data_root,
-                        "sos_commands/process/ps_axo_flags_state_"
-                        "uid_pid_ppid_pgid_sid_cls_pri_addr_sz_wchan*_lstart_"
-                        "tty_time_cmd")
-    _paths = []
-    for path in glob.glob(path):
-        _paths.append(path)
-
-    if not _paths:
-        return
-
-    # strip data_root since it will be prepended later
-    return _paths[0].partition(HotSOSConfig.data_root)[2]
+        return None
