@@ -1,6 +1,7 @@
 import abc
 import re
 import subprocess
+from dataclasses import dataclass
 
 from hotsos.core.factory import FactoryBase
 from hotsos.core.host_helpers.cli import CLIHelper
@@ -9,18 +10,19 @@ from hotsos.core.utils import sorted_dict
 
 
 class DPKGBadVersionSyntax(Exception):
-    pass
+    """ Exception raised when an invalid version is provided in a check. """
 
 
-class DPKGVersion(object):
-
+class DPKGVersion():
+    """ Helper for querying and comparing dpkg packaging versions. """
     def __init__(self, a):
         self.a = str(a)
 
     def _compare_impl(self, op, b):
         try:
             output = subprocess.check_output(['dpkg', '--compare-versions',
-                                   self.a, op, b], stderr=subprocess.STDOUT)
+                                              self.a, op, b],
+                                             stderr=subprocess.STDOUT)
             if re.search(b"dpkg: warning: version.*has bad syntax:.*", output):
                 raise DPKGBadVersionSyntax(output)
         except subprocess.CalledProcessError as se:
@@ -31,6 +33,9 @@ class DPKGVersion(object):
 
         return True
 
+    def __repr__(self):
+        return str(self)
+
     def __str__(self):
         return self.a
 
@@ -49,9 +54,166 @@ class DPKGVersion(object):
     def __ge__(self, b):
         return self._compare_impl('ge', str(b))
 
+    @staticmethod
+    def normalize_version_criteria(version_criteria):
+        """Normalize all the criterions in a criteria.
+
+        Normalization does the following:
+        - removes empty criteria
+        - replaces old ops with the new ones
+        - sorts each criterion(ascending) and criteria(descending)
+        - adds upper/lower bounds to criteria, where needed
+
+        @param version_criteria: List of version ranges to normalize
+        @return: Normalized list of version ranges
+        """
+
+        # Step 0: Ensure that all version values are DPKGVersion type
+        for idx, version_criterion in enumerate(version_criteria):
+            for k, v in version_criterion.items():
+                version_criterion.update({k: DPKGVersion(v)})
+
+        # Step 1: Remove empty criteria
+        version_criteria = [x for x in version_criteria if len(x) > 0]
+
+        # Step 2: Replace legacy ops with the new ones
+        legacy_ops = {"min": "ge", "max": "le"}
+        for idx, version_criterion in enumerate(version_criteria):
+            for lop, nop in legacy_ops.items():
+                if lop in version_criterion:
+                    version_criterion[nop] = version_criterion[lop]
+                    del version_criterion[lop]
+
+        # Step 3: Sort each criterion in itself, so the smallest version
+        # appears first
+        for idx, version_criterion in enumerate(version_criteria):
+            version_criterion = dict(sorted(version_criterion.items(),
+                                     key=lambda a: a[1]))
+            version_criteria[idx] = version_criterion
+
+        # Step 4: Sort all criteria by the first element in the criterion
+        version_criteria = sorted(version_criteria,
+                                  key=lambda a: list(a.values())[0])
+
+        # Step 5: Add the implicit upper/lower bounds where needed
+        lower_bound_ops = ["gt", "ge", "eq"]  # ops that define a lower bound
+        upper_bound_ops = ["lt", "le", "eq"]  # ops that define an upper bound
+        equal_compr_ops = ["eq", "ge", "le"]  # ops that compare for equality
+        for idx, version_criterion in enumerate(version_criteria):
+            log.debug("\tchecking criterion %s", str(version_criterion))
+
+            has_lower_bound = any(x in lower_bound_ops
+                                  for x in version_criterion)
+            has_upper_bound = any(x in upper_bound_ops
+                                  for x in version_criterion)
+            is_the_last_item = idx == (len(version_criteria) - 1)
+            is_the_first_item = idx == 0
+
+            log.debug("\t\tcriterion %s has lower bound?"
+                      "%s has upper bound? %s", str(version_criterion),
+                      has_lower_bound, has_upper_bound)
+
+            if not has_upper_bound and not is_the_last_item:
+                op = "le"  # default
+                next_criterion = version_criteria[idx + 1]
+                next_op, next_val = list(next_criterion.items())[0]
+                # If the next criterion op compares for equality, then the
+                # implicit op added to this criterion should not compare for
+                # equality.
+                if next_op in equal_compr_ops:
+                    op = "lt"
+                log.debug("\t\tadding implicit upper bound %s:%s to %s", op,
+                          next_val, version_criterion)
+                version_criterion[op] = next_val
+            elif not has_lower_bound and not is_the_first_item:
+                op = "ge"  # default
+                prev_criterion = version_criteria[idx - 1]
+                prev_op, prev_val = list(prev_criterion.items())[-1]
+                # If the previous criterion op compares for equality, then the
+                # implicit op added to this criterion should not compare for
+                # equality.
+                if prev_op in equal_compr_ops:
+                    op = "gt"
+                log.debug("\t\tadding implicit lower bound %s:%s to %s", op,
+                          prev_val, version_criterion)
+                version_criterion[op] = prev_val
+
+            # Re-sort and overwrite the criterion
+            version_criteria[idx] = dict(
+                sorted(version_criterion.items(),
+                       key=lambda a: a[1]))
+
+        # Step 6: Sort by descending order so the largest version range
+        # appears first
+        version_criteria = sorted(version_criteria,
+                                  key=lambda a: list(a.values())[0],
+                                  reverse=True)
+
+        log.debug("final criteria: %s", str(version_criteria))
+        return version_criteria
+
+    @staticmethod
+    def is_version_within_ranges(version, version_criteria):
+        """Check if pkg's version satisfies any criterion listed in
+        the version_criteria.
+
+        @param version: Version
+        @param version_criteria: Criteria to check version against
+
+        @return: True if ver(pkg) satisfies any criterion, false otherwise.
+        """
+        result = True
+
+        # Supported operations for defining version ranges
+        ops = {
+            "eq": lambda lhs, rhs: lhs == DPKGVersion(rhs),
+            "lt": lambda lhs, rhs: lhs < DPKGVersion(rhs),
+            "le": lambda lhs, rhs: lhs <= DPKGVersion(rhs),
+            "gt": lambda lhs, rhs: lhs > DPKGVersion(rhs),
+            "ge": lambda lhs, rhs: lhs >= DPKGVersion(rhs),
+            # Here, it's necessary to use lambda. It defers the
+            # "ops" access to runtime, so ops["ge"] exists when
+            # this is executed.
+            # pylint: disable-next=unnecessary-lambda
+            "min": lambda lhs, rhs: ops["ge"](lhs, rhs),
+            # pylint: disable-next=unnecessary-lambda
+            "max": lambda lhs, rhs: ops["le"](lhs, rhs),
+        }
+
+        version_criteria = DPKGVersion.normalize_version_criteria(
+            version_criteria)
+
+        for version_criterion in version_criteria:
+            if not version_criterion.keys() <= ops.keys():
+                raise NameError(
+                    "Unrecognized comparison operator name(s):"
+                    f"{version_criterion.keys()-ops.keys()}")
+            # Each criterion is evaluated on its own
+            # so if any of the criteria is true, then
+            # the check is also true.
+            for op_name, op_fn in ops.items():
+                if op_name in version_criterion:
+                    cversion = str(version_criterion[op_name])
+                    # Check if the criterion is satisfied or not
+                    if not op_fn(str(version), cversion):
+                        break
+            else:
+                # Loop is not exited by a break which means
+                # all ops in the criterion are satisfied.
+                result = True
+                log.debug("version criterion %s satisfied by %s",
+                          version_criterion, version)
+                # Break the outer loop
+                break
+            result = False
+
+        log.debug("version %s within version ranges %s "
+                  "(result=%s)", version, version_criteria, result)
+        return result
+
 
 class PackageHelperBase(abc.ABC):
-
+    """ Base class for packaging helpers. """
     def get_version(self, pkg):
         """
         Return version of package.
@@ -75,7 +237,7 @@ class PackageHelperBase(abc.ABC):
         if not _all:
             return []
 
-        return ["{} {}".format(*e) for e in _all.items()]
+        return [f"{e[0]} {e[1]}" for e in _all.items()]
 
     @property
     def all(self):
@@ -89,7 +251,7 @@ class PackageHelperBase(abc.ABC):
 
 
 class DockerImageHelper(PackageHelperBase):
-
+    """ Helpers for analysing docker images. """
     def __init__(self, core_pkgs, other_pkgs=None):
         """
         @param core_pkgs:
@@ -137,7 +299,7 @@ class DockerImageHelper(PackageHelperBase):
         used_images = self.get_container_images()
         image_list = self.cli.docker_images()
         if not image_list:
-            return
+            return None
 
         all_exprs = self.core_image_exprs + self.other_image_exprs
         for line in image_list:
@@ -183,12 +345,12 @@ class DockerImageHelper(PackageHelperBase):
             return {}
 
         # go fetch
-        self.all
+        _ = self.all
         return self._core_images
 
 
 class APTPackageHelper(PackageHelperBase):
-
+    """ Helpers for analysing apt packages. """
     def __init__(self, core_pkgs, other_pkgs=None):
         """
         @param core_pkgs: list of python.re expressions used to match
@@ -226,7 +388,7 @@ class APTPackageHelper(PackageHelperBase):
         if not dpkg_l:
             return False
 
-        cexpr = re.compile(r"^.i\s+{}\s+.+".format(pkg))
+        cexpr = re.compile(rf"^.i\s+{pkg}\s+.+")
         for line in dpkg_l:
             # See https://man7.org/linux/man-pages/man1/dpkg-query.1.html for
             # package states.
@@ -248,6 +410,8 @@ class APTPackageHelper(PackageHelperBase):
                 if name:
                     return version
 
+        return None
+
     def _match_package(self, pkg, entry):
         """ Returns tuple of (package name, version) """
         expr = self._match_expr_template.format(pkg)
@@ -266,14 +430,12 @@ class APTPackageHelper(PackageHelperBase):
         dpkg_l = self.cli.dpkg_l()
         if not dpkg_l:
             return self._all_packages
-
         all_exprs = self.core_pkg_exprs + self.other_pkg_exprs
         for line in dpkg_l:
             for pkg in all_exprs:
                 name, version = self._match_package(pkg, line)
                 if name is None:
                     continue
-
                 if pkg in self.core_pkg_exprs:
                     self._core_packages[name] = version
                 else:
@@ -311,15 +473,15 @@ class APTPackageHelper(PackageHelperBase):
             return self._core_packages
 
         # go fetch
-        self.all
+        _ = self.all
         return self._core_packages
 
 
-class AptPackage(object):
-
-    def __init__(self, name, version):
-        self.name = name
-        self.version = version
+@dataclass(frozen=True)
+class AptPackage:
+    """ Representation of an APT package.  """
+    name: str
+    version: str
 
 
 class AptFactory(FactoryBase):
@@ -336,9 +498,11 @@ class AptFactory(FactoryBase):
         if name in helper.all:
             return AptPackage(name, helper.all[name])
 
+        return None
+
 
 class SnapPackageHelper(PackageHelperBase):
-
+    """ Helpers for analysing snap packages. """
     def __init__(self, core_snaps, other_snaps=None):
         """
         @param core_snaps: list of python.re expressions used to match
@@ -358,7 +522,8 @@ class SnapPackageHelper(PackageHelperBase):
     def is_installed(self, pkg):
         return pkg in self.all
 
-    def _get_snap_info_from_line(self, line, cexpr):
+    @staticmethod
+    def _get_snap_info_from_line(line, cexpr):
         """ Returns snap name and info if found in line. """
         ret = re.match(cexpr, line)
         if ret:
@@ -368,7 +533,7 @@ class SnapPackageHelper(PackageHelperBase):
                     'channel': ret.group(4),
                     }
 
-        return
+        return None
 
     def get_revision(self, snap):
         """ Return revision of package.
@@ -377,6 +542,8 @@ class SnapPackageHelper(PackageHelperBase):
         if info:
             return info[0]['revision']
 
+        return None
+
     def get_version(self, pkg):
         """ Return version of snap package.
 
@@ -386,6 +553,8 @@ class SnapPackageHelper(PackageHelperBase):
         if info:
             return info[0]['version']
 
+        return None
+
     def get_channel(self, pkg):
         """ Return channel of snap package.
 
@@ -395,6 +564,8 @@ class SnapPackageHelper(PackageHelperBase):
         if info:
             return info[0]['channel']
 
+        return None
+
     def _get_snap_info(self, snap_name_expr):
         """
         Return a list of info for snaps matched using the expression.
@@ -404,7 +575,7 @@ class SnapPackageHelper(PackageHelperBase):
         @return: a list of snaps and their info.
         """
         if not self.snap_list_all:
-            return
+            return None
 
         info = []
         cexpr = re.compile(self._match_expr_template.format(snap_name_expr))
@@ -472,8 +643,7 @@ class SnapPackageHelper(PackageHelperBase):
         if not _all:
             return []
 
-        return ["{} {}".format(name, info['version'])
-                for name, info in _all.items()]
+        return [f"{name} {info['version']}" for name, info in _all.items()]
 
     @property
     def core(self):
@@ -487,5 +657,5 @@ class SnapPackageHelper(PackageHelperBase):
             return {}
 
         # go fetch
-        self.all
+        _ = self.all
         return self._core_snaps
