@@ -35,8 +35,8 @@ class SYSCtlFactory(FactoryBase):
         return self.get(name)
 
 
-class SYSCtlConfHelper(object):
-
+class SYSCtlConfHelper():
+    """ Helpers used to analyse sysctl config files. """
     def __init__(self, path):
         self.path = path
         self._config = {}
@@ -68,7 +68,7 @@ class SYSCtlConfHelper(object):
 
         setters = {}
         unsetters = {}
-        with open(self.path) as fd:
+        with open(self.path, encoding='utf-8') as fd:
             for line in fd.readlines():
                 if line.startswith("#"):
                     continue
