@@ -5,7 +5,8 @@ from hotsos.core.host_helpers import CLIHelper
 from hotsos.core.log import log
 
 
-class UptimeHelper(object):
+class UptimeHelper():
+    """ Helper to enable querying system uptime. """
     def __init__(self):
         """
         uptime is either expressed in different combinations of days, hours
@@ -18,8 +19,7 @@ class UptimeHelper(object):
         # this needs to take into account the different formats supported by
         # https://gitlab.com/procps-ng/procps/-/blob/newlib/library/uptime.c
         etime_expr = r"(?:([\d:]+)|(\d+\s+\S+,\s+[\d:]+)|(\d+\s+\S+)),"
-        expr = (r"\s*[\d:]+\s+up\s+{}.+\s+load average:\s+(.+)".
-                format(etime_expr))
+        expr = rf"\s*[\d:]+\s+up\s+{etime_expr}.+\s+load average:\s+(.+)"
         ret = re.compile(expr).match(self.uptime)
         self.subgroups = {}
         if ret:
@@ -36,7 +36,7 @@ class UptimeHelper(object):
         """ Total uptime in minutes. """
         if not self.subgroups:
             log.info("uptime not available")
-            return
+            return None
 
         if self.subgroups['hour']['value']:
             expr = self.subgroups['hour']['expr']
@@ -84,9 +84,11 @@ class UptimeHelper(object):
         days = int(self.in_hours / 24)
         hours = self.in_hours - days * 24
         minutes = self.in_minutes - (self.in_hours * 60)
-        return "{}d:{}h:{}m".format(days, hours, minutes)
+        return f"{days}d:{hours}h:{minutes}m"
 
     @property
     def loadavg(self):
         if self.subgroups:
             return self.subgroups['loadavg']['value']
+
+        return None
