@@ -1,2 +1,2 @@
-from .common import JujuChecksBase  # noqa: F403,F401
+from .common import JujuChecks  # noqa: F403,F401
 from .resources import JujuBinaryInterface  # noqa: F403,F401
