@@ -5,14 +5,15 @@ from hotsos.core.plugins.juju.resources import JujuBase
 from hotsos.core import plugintools
 
 SVC_VALID_SUFFIX = r'[0-9a-zA-Z-_]*'
-JUJU_SVC_EXPRS = [r'mongod{}'.format(SVC_VALID_SUFFIX),
-                  r'jujud{}'.format(SVC_VALID_SUFFIX),
+JUJU_SVC_EXPRS = [rf'mongod{SVC_VALID_SUFFIX}',
+                  rf'jujud{SVC_VALID_SUFFIX}',
                   # catch juju-db but filter out processes with juju-db in
                   # their args list.
-                  r'(?:^|[^\s])juju-db{}'.format(SVC_VALID_SUFFIX)]
+                  rf'(?:^|[^\s])juju-db{SVC_VALID_SUFFIX}']
 
 
-class JujuChecksBase(plugintools.PluginPartBase, JujuBase):
+class JujuChecks(plugintools.PluginPartBase, JujuBase):
+    """ Juju checks. """
     plugin_name = 'juju'
     plugin_root_index = 12
 
@@ -23,6 +24,13 @@ class JujuChecksBase(plugintools.PluginPartBase, JujuBase):
         # this is needed for juju scenarios
         self.systemd_processes = self.systemd.processes
 
+    @property
+    def version(self):
+        if self.machine:
+            return self.machine.version
+
+        return "unknown"
+
     @property
     def plugin_runnable(self):
         return os.path.exists(self.juju_lib_path)
