@@ -7,8 +7,8 @@ from hotsos.core.config import HotSOSConfig
 from hotsos.core.plugins.kernel.config import KernelConfig
 
 
-class KernelBase(object):
-
+class KernelBase():
+    """ Base class for kernel plugin helpers. """
     @cached_property
     def version(self):
         """Returns string kernel version."""
@@ -30,7 +30,7 @@ class KernelBase(object):
         parameters = []
         path = os.path.join(HotSOSConfig.data_root, "proc/cmdline")
         if os.path.exists(path):
-            with open(path) as fd:
+            with open(path, encoding='utf-8') as fd:
                 cmdline = fd.read().strip()
             for entry in cmdline.split():
                 if entry.startswith("BOOT_IMAGE"):
@@ -43,8 +43,11 @@ class KernelBase(object):
 
             return parameters
 
+        return None
 
-class KernelChecksBase(KernelBase, plugintools.PluginPartBase):
+
+class KernelChecks(KernelBase, plugintools.PluginPartBase):
+    """ Base class for all kernel checks. """
     plugin_name = 'kernel'
     plugin_root_index = 14
 
