@@ -3,15 +3,17 @@ import os
 
 from hotsos.core.config import HotSOSConfig
 from hotsos.core.host_helpers import CLIHelper, HostNetworkingHelper
-from hotsos.core.search import FileSearcher, SearchConstraintSearchSince
-from hotsos.core.ycheck.engine.properties.search import CommonTimestampMatcher
+from hotsos.core.search import (
+    CommonTimestampMatcher,
+    FileSearcher,
+    SearchConstraintSearchSince,
+)
 
 KERNLOG_TS = r'\[\s*\d+\.\d+\]'
-KERNLOG_PREFIX = (r'(?:\S+\s+\d+\s+[\d:]+\s+\S+\s+\S+:\s+)?{}'.
-                  format(KERNLOG_TS))
+KERNLOG_PREFIX = rf'(?:\S+\s+\d+\s+[\d:]+\s+\S+\s+\S+:\s+)?{KERNLOG_TS}'
 
 
-class CallTraceHeuristicBase(object):
+class CallTraceHeuristicBase():
     """ Defines a common interface for heuristics implementations. """
 
     @abc.abstractmethod
@@ -19,7 +21,7 @@ class CallTraceHeuristicBase(object):
         """ Ensure callable to get results. """
 
 
-class CallTraceStateBase(object):
+class CallTraceStateBase():
     """
     A state capture object that allows getting and setting arbitrary state.
     """
@@ -80,8 +82,8 @@ class TraceTypeBase(abc.ABC):
         """ Iterate over each call trace found. """
 
 
-class KernLogBase(object):
-
+class KernLogBase():
+    """ Base class for kernlog analysis implementations. """
     def __init__(self):
         c = SearchConstraintSearchSince(ts_matcher_cls=CommonTimestampMatcher)
         self.searcher = FileSearcher(constraint=c)
@@ -92,6 +94,6 @@ class KernLogBase(object):
     def path(self):
         path = os.path.join(HotSOSConfig.data_root, 'var/log/kern.log')
         if HotSOSConfig.use_all_logs:
-            return "{}*".format(path)
+            return f"{path}*"
 
         return path
