@@ -36,13 +36,13 @@ class ProcNetBase(abc.ABC):
 
         return round((count * 100.0) / total, 2)
 
-    def _process_file(self, fname):
+    def process_file(self, fname):
         if not os.path.exists(fname):
             log.debug("file not found '%s' - skipping load", fname)
             return
 
         log.debug("start processing %s", fname)
-        with open(fname) as fd:
+        with open(fname, encoding='utf-8') as fd:
             labels = None
             values = None
             for line in fd:
@@ -97,28 +97,27 @@ class ProcNetBase(abc.ABC):
         if fld in self._fields:
             return self._get_field(self._header, fld)
 
-        raise AttributeError("{} not found in {}".
-                             format(fld, self.__class__.__name__))
+        raise AttributeError(f"{fld} not found in {self.__class__.__name__}")
 
 
 class SNMPBase(ProcNetBase):
-
+    """ Base class for /proc/net/snmp interface implementations. """
     def __init__(self):
         super().__init__()
-        self._process_file(os.path.join(HotSOSConfig.data_root,
-                                        'proc/net/snmp'))
+        self.process_file(os.path.join(HotSOSConfig.data_root,
+                                       'proc/net/snmp'))
 
 
 class SNMPTcp(SNMPBase):
-
-    def PcentInSegs(self, field):
+    """ /proc/net/snmp interface implementation to extract TCP information. """
+    def _percent_in_segs(self, field):
         """
         The value of a field can be provided as a percentage of the total rx
         segments.
         """
         return self._pcent_of(field, self.InSegs)
 
-    def PcentOutSegs(self, field):
+    def _percent_out_segs(self, field):
         """
         The value of a field can be provided as a percentage of the total tx
         segments.
@@ -146,28 +145,28 @@ class SNMPTcp(SNMPBase):
         """
         if fld.endswith('PcentInSegs'):
             fld = fld.partition('PcentInSegs')[0]
-            return self.PcentInSegs(fld)
+            return self._percent_in_segs(fld)
         if fld.endswith('PcentOutSegs'):
             fld = fld.partition('PcentOutSegs')[0]
-            return self.PcentOutSegs(fld)
+            return self._percent_out_segs(fld)
 
         return super().__getattr__(fld)
 
 
 class SNMPUdp(SNMPBase):
-
+    """ /proc/net/snmp interface implementation to extract UDP information. """
     @property
     def _header(self):
         return 'Udp'
 
-    def PcentInDatagrams(self, field):
+    def _percent_in_datagrams(self, field):
         """
         The value of a field can be provided as a percentage of the total rx
         datagrams.
         """
         return self._pcent_of(field, self.InDatagrams)
 
-    def PcentOutDatagrams(self, field):
+    def _percent_out_datagrams(self, field):
         """
         The value of a field can be provided as a percentage of the total tx
         datagrams.
@@ -191,37 +190,39 @@ class SNMPUdp(SNMPBase):
         """
         if fld.endswith('PcentInDatagrams'):
             fld = fld.partition('PcentInDatagrams')[0]
-            return self.PcentInDatagrams(fld)
+            return self._percent_in_datagrams(fld)
         if fld.endswith('PcentOutDatagrams'):
             fld = fld.partition('PcentOutDatagrams')[0]
-            return self.PcentOutDatagrams(fld)
+            return self._percent_out_datagrams(fld)
 
         return super().__getattr__(fld)
 
 
 class NetStatBase(ProcNetBase):
-
+    """ Base class for /proc/net/netstat implementations.  """
     def __init__(self):
         super().__init__()
-        self._process_file(os.path.join(HotSOSConfig.data_root,
-                                        'proc/net/netstat'))
+        self.process_file(os.path.join(HotSOSConfig.data_root,
+                                       'proc/net/netstat'))
         self.net_snmp_tcp = SNMPTcp()
 
 
 class NetStatTCP(NetStatBase):
-
+    """
+    /proc/net/netstat interface implementation to extract TCP information.
+    """
     @property
     def _header(self):
         return 'TcpExt'
 
-    def PcentInSegs(self, field):
+    def _percent_in_segs(self, field):
         """
         The value of a field can be provided as a percentage of the total rx
         segments.
         """
         return self._pcent_of(field, self.net_snmp_tcp.InSegs)
 
-    def PcentOutSegs(self, field):
+    def _percent_out_segs(self, field):
         """
         The value of a field can be provided as a percentage of the total tx
         segments.
@@ -278,16 +279,15 @@ class NetStatTCP(NetStatBase):
         """
         if fld.endswith('PcentInSegs'):
             fld = fld.partition('PcentInSegs')[0]
-            return self.PcentInSegs(fld)
+            return self._percent_in_segs(fld)
         if fld.endswith('PcentOutSegs'):
             fld = fld.partition('PcentOutSegs')[0]
-            return self.PcentOutSegs(fld)
+            return self._percent_out_segs(fld)
 
         return super().__getattr__(fld)
 
 
 class SockStat(ProcNetBase):
-
     """
     Provides a common way to extract fields from /proc/net/sockstat.
 
@@ -311,10 +311,7 @@ class SockStat(ProcNetBase):
         self._maybe_parse_sysctl_net_ipv4_xmem("UDP")
 
     def _maybe_parse_sysctl_net_ipv4_xmem(self, key):
-        xmem = getattr(
-            SYSCtlFactory(),
-            'net.ipv4.{}_mem'.format(key.lower())
-        )
+        xmem = getattr(SYSCtlFactory(), f'net.ipv4.{key.lower()}_mem')
         # Some environments may not have all the sysctl
         # parameters present, so make parsing `optional`
         if xmem is not None:
@@ -331,10 +328,10 @@ class SockStat(ProcNetBase):
     def _process_file(self, fname):
         if not os.path.exists(fname):
             log.debug("file not found '%s' - skipping load", fname)
-            return
+            return None
 
         log.debug("start processing %s", fname)
-        with open(fname) as fd:
+        with open(fname, encoding='utf-8') as fd:
             for line in fd:
                 psl = line.split(':')
                 if len(psl) != 2:
@@ -350,12 +347,14 @@ class SockStat(ProcNetBase):
                     # Convert string values to `int``
                     stats = {k: int(v) for k, v in stats.items()}
                     self._data[label] = stats
-                except Exception as e:
+                except ValueError as e:
                     log.warning("badness: failed to parse statistics "
                                 "for `%s`, bad data `%s` (%s)",
                                 label, stats, e)
                     continue
 
+        return None
+
     @property
     def _header(self):
         # We do not expect this to be used
@@ -396,14 +395,13 @@ class SockStat(ProcNetBase):
                 self._fields[fld][1]
             )
 
-        raise AttributeError("{} not found in {}".
-                             format(fld, self.__class__.__name__))
+        raise AttributeError(f"{fld} not found in {self.__class__.__name__}")
 
 
 class STOVParserBase(UserList):
-
+    """ Base class for implementations of Standard Table of Values. """
     def __init__(self):
-        self.data = []
+        super().__init__()
         self._load()
 
     @abc.abstractmethod
@@ -438,8 +436,8 @@ class STOVParserBase(UserList):
         """
 
 
-class FieldTranslator(object):
-
+class FieldTranslator():
+    """ Field translator. """
     def __init__(self, result, translations):
         self.result = result
         self.translations = translations
@@ -476,13 +474,15 @@ class Lsof(STOVParserBase):
             for r in results.find_by_tag('content'):
                 self.data.append(r)
 
-    def _int_if_numeric(self, value):
+    @staticmethod
+    def _int_if_numeric(value):
         if value.isnumeric():
             return int(value)
 
         return value
 
-    def _strip(self, value):
+    @staticmethod
+    def _strip(value):
         return value.strip()
 
     @property
@@ -515,7 +515,7 @@ class Lsof(STOVParserBase):
 
             # These fields can be empty/None
             if key in ['TYPE', 'DEVICE', 'SIZE/OFF', 'NODE']:
-                _expr = '{}?'.format(_expr)
+                _expr = f'{_expr}?'
 
             expr.append(_expr)
 
@@ -619,7 +619,8 @@ class NetLink(STOVParserBase):
     @property
     def all_with_drops_str(self):
         if not self.all_with_drops:
-            return
+            return None
+
         drops_info = [
             f"\tnlsock inode `{nlsock.sk_inode_num}`: "
             f"procs[{','.join(nlsock.procs)}]"
