@@ -6,9 +6,10 @@ from hotsos.core import host_helpers
 from hotsos.core.plugins.system.system import SystemBase
 
 
-class SYSFSBase(object):
-
-    def get(self, relpath):
+class SYSFSBase():
+    """ Base class for sysfs interface implementations. """
+    @staticmethod
+    def get(relpath):
         """
         Read a sysfs entry and return its value.
 
@@ -16,36 +17,38 @@ class SYSFSBase(object):
         """
         path = os.path.join(HotSOSConfig.data_root, 'sys', relpath)
         if not os.path.exists(path):
-            return
+            return None
 
-        with open(path) as fd:
-            value = fd.read()
-
-        return value.strip()
+        with open(path, encoding='utf-8') as fd:
+            return fd.read().strip()
 
 
 class CPU(SYSFSBase):
-
+    """ Helper to get CPU information. """
     @property
     def model(self):
         out = host_helpers.CLIHelper().lscpu()
-        if not out:
-            return
+        if out:
+            for line in out:
+                if not line.startswith("Model name:"):
+                    continue
 
-        for line in out:
-            if line.startswith("Model name:"):
                 return re.search(r'Model name:\s+(.+)', line).group(1)
 
+        return None
+
     @property
     def vendor(self):
         out = host_helpers.CLIHelper().lscpu()
-        if not out:
-            return
+        if out:
+            for line in out:
+                if not line.startswith("Vendor ID:"):
+                    continue
 
-        for line in out:
-            if line.startswith("Vendor ID:"):
                 return re.search(r'Vendor ID:\s+(.+)', line).group(1).lower()
 
+        return None
+
     @property
     def isolated(self):
         """ This means that isolcpus is configured. """
@@ -54,17 +57,11 @@ class CPU(SYSFSBase):
     @property
     def smt(self):
         smt = self.get('devices/system/cpu/smt/active')
-        if smt is None:
-            return
-
-        if smt == '1':
-            return True
-
-        return False
+        return smt == '1'
 
     def cpufreq_scaling_governor(self, cpu_id):
-        return self.get('devices/system/cpu/cpu{}/cpufreq/scaling_governor'.
-                        format(cpu_id))
+        return self.get(f'devices/system/cpu/cpu{cpu_id}/cpufreq/'
+                        'scaling_governor')
 
     @property
     def cpufreq_scaling_governor_all(self):
