@@ -14,7 +14,8 @@ CORE_SNAPS = CORE_APT
 SERVICE_EXPRS = [s + '[A-Za-z0-9-]*' for s in CORE_APT + APT_DEPS]
 
 
-class MAASChecksBase(PluginPartBase):
+class MAASChecks(PluginPartBase):
+    """ MAAS checks. """
     plugin_name = 'maas'
     plugin_root_index = 13
 
