@@ -1,30 +1,55 @@
 import os
 from datetime import datetime
 from functools import cached_property
+from dataclasses import dataclass, field
 
 from hotsos.core.config import HotSOSConfig
 from hotsos.core import host_helpers
 from hotsos.core.log import log
 from hotsos.core.plugins.openstack.exceptions import (
     EXCEPTIONS_COMMON,
+)
+from hotsos.core.plugins.openstack.exceptions_barbican import (
     BARBICAN_EXCEPTIONS,
+)
+from hotsos.core.plugins.openstack.exceptions_castellan import (
     CASTELLAN_EXCEPTIONS,
+)
+from hotsos.core.plugins.openstack.exceptions_cinder import (
     CINDER_EXCEPTIONS,
+)
+from hotsos.core.plugins.openstack.exceptions_designate import (
     DESIGNATE_EXCEPTIONS,
+)
+from hotsos.core.plugins.openstack.exceptions_glance import (
     GLANCE_EXCEPTIONS,
     GLANCE_STORE_EXCEPTIONS,
+)
+from hotsos.core.plugins.openstack.exceptions_heat import (
     HEAT_EXCEPTIONS,
+)
+from hotsos.core.plugins.openstack.exceptions_keystone import (
     KEYSTONE_EXCEPTIONS,
+)
+from hotsos.core.plugins.openstack.exceptions_manila import (
     MANILA_EXCEPTIONS,
-    PLACEMENT_EXCEPTIONS,
-    PYTHON_LIBVIRT_EXCEPTIONS,
-    NOVA_EXCEPTIONS,
+)
+from hotsos.core.plugins.openstack.exceptions_masakari import (
+    MASAKARI_EXCEPTIONS,
+)
+from hotsos.core.plugins.openstack.exceptions_neutron import (
     NEUTRON_EXCEPTIONS,
     NEUTRONCLIENT_EXCEPTIONS,
-    OCTAVIA_EXCEPTIONS,
-    OS_VIF_EXCEPTIONS,
     OVSDBAPP_EXCEPTIONS,
-    MASAKARI_EXCEPTIONS,
+)
+from hotsos.core.plugins.openstack.exceptions_nova import (
+    NOVA_EXCEPTIONS,
+    OS_VIF_EXCEPTIONS,
+    PLACEMENT_EXCEPTIONS,
+    PYTHON_LIBVIRT_EXCEPTIONS,
+)
+from hotsos.core.plugins.openstack.exceptions_octavia import (
+    OCTAVIA_EXCEPTIONS,
 )
 
 
@@ -270,21 +295,56 @@ OST_EXCEPTIONS = {'barbican': BARBICAN_EXCEPTIONS + CASTELLAN_EXCEPTIONS,
                   }
 
 
-class OpenstackConfig(host_helpers.SectionalConfigBase):
-
+class OpenstackConfig(host_helpers.IniConfigBase):
+    """ Openstack config interface. """
     def __getattr__(self, key):
         return self.get(key)
 
 
-class OSTProject(object):
+@dataclass(frozen=True)
+class OSTProjectParameters:
+    """Parameters for OSTProject class.
+
+        @param name: name of this project
+        @param config: dict of config files keyed by a label used to identify
+                       them. All projects should have a config file labelled
+                       'main'.
+        @param apt_core_alt: optional list of apt packages (regex) that are
+                             used by this project where the name of the project
+                             is not the same as the name used for its packages.
+        @param systemd_masked_services: optional list of services that are
+               expected to be masked in systemd e.g. if they are actually being
+               run by apache.
+        @param systemd_deprecated_services: optional list of services that are
+               deprecated. This can be helpful if e.g. you want to skip checks
+               for resources related to these services.
+        @param systemd_extra_services: optional list of systemd services that
+               are used. This is useful e.g. if components are run under
+               apache or if a package runs components using services whose name
+               don't match the name of the project.
+        @param log_path_overrides: specify log path for a given service that
+                                   overrides the default format i.e.
+                                   /var/log/<project>/<svc>.log.
+    """
+    name: str = None
+    config: dict = None
+    apt_core_alt: list = None
+    systemd_masked_services: list = field(default_factory=lambda: [])
+    systemd_deprecated_services: list = field(default_factory=lambda: [])
+    systemd_extra_services: list = field(default_factory=lambda: [])
+    log_path_overrides: dict = field(default_factory=lambda: {})
+
+
+class OSTProject():
+    """
+    Representation of an Openstack project/service.
+
+    Provides service state, version, packages etc.
+    """
     SVC_VALID_SUFFIX = r'[0-9a-zA-Z-_]*'
     PY_CLIENT_PREFIX = r"python3?-{}\S*"
 
-    def __init__(self, name, config=None, apt_core_alt=None,
-                 systemd_masked_services=None,
-                 systemd_deprecated_services=None,
-                 log_path_overrides=None,
-                 systemd_extra_services=None):
+    def __init__(self, params: OSTProjectParameters):
         """
         @param name: name of this project
         @param config: dict of config files keyed by a label used to identify
@@ -307,26 +367,34 @@ class OSTProject(object):
                apache or if a package runs components using services whose name
                don't match the name of the project.
         """
-        self.name = name
-        self.packages_deps = [self.PY_CLIENT_PREFIX.format(name)]
-        self.packages_core = [name]
-        if apt_core_alt:
-            self.packages_core.extend(apt_core_alt)
-            for c in apt_core_alt:
+        self.name = params.name
+        self.packages_deps = [self.PY_CLIENT_PREFIX.format(params.name)]
+        self.packages_core = [params.name]
+        if params.apt_core_alt:
+            self.packages_core.extend(params.apt_core_alt)
+            for c in params.apt_core_alt:
                 self.packages_deps.append(self.PY_CLIENT_PREFIX.format(c))
 
         self.config = {}
-        if config:
-            for label, path in config.items():
-                path = os.path.join(HotSOSConfig.data_root, 'etc', name, path)
+        if params.config:
+            for label, path in params.config.items():
+                path = os.path.join(
+                    HotSOSConfig.data_root,
+                    "etc",
+                    params.name,
+                    path
+                )
                 self.config[label] = OpenstackConfig(path)
 
-        self.systemd_extra_services = systemd_extra_services or []
-        self.systemd_masked_services = systemd_masked_services or []
-        self.systemd_deprecated_services = systemd_deprecated_services or []
-        self.logs_path = os.path.join('var/log', name)
-        self.log_path_overrides = log_path_overrides or {}
-        self.exceptions = EXCEPTIONS_COMMON + OST_EXCEPTIONS.get(name, [])
+        self.systemd_extra_services = params.systemd_extra_services
+        self.systemd_masked_services = params.systemd_masked_services
+        self.systemd_deprecated_services = params.systemd_deprecated_services
+        self.logs_path = os.path.join("var/log", params.name)
+        self.log_path_overrides = params.log_path_overrides
+        self.exceptions = EXCEPTIONS_COMMON + OST_EXCEPTIONS.get(
+            params.name,
+            []
+        )
 
     @cached_property
     def installed(self):
@@ -336,7 +404,7 @@ class OSTProject(object):
 
     @cached_property
     def services_expr(self):
-        exprs = ['{}{}'.format(self.name, self.SVC_VALID_SUFFIX)]
+        exprs = [f'{self.name}{self.SVC_VALID_SUFFIX}']
         if self.systemd_extra_services:
             exprs += self.systemd_extra_services
         return exprs
@@ -356,8 +424,8 @@ class OSTProject(object):
         """
         Returns tuples of daemon name, log path for each agent/daemon.
         """
-        proj_manage = "{}-manage".format(self.name)
-        path = os.path.join('var/log', self.name, "{}.log".format(proj_manage))
+        proj_manage = f"{self.name}-manage"
+        path = os.path.join('var/log', self.name, f"{proj_manage}.log")
         yield proj_manage, [path]
         for svc in self.services:
             if (not include_deprecated_services and
@@ -365,17 +433,18 @@ class OSTProject(object):
                 continue
 
             path = os.path.join('var/log', self.name,
-                                "{}.log".format(svc))
+                                f"{svc}.log")
             yield svc, self.log_path_overrides.get(svc, [path])
 
 
-class OSTProjectCatalog(object):
+class OSTProjectCatalog():
+    """ Catalog of Openstack projects tracked by this plugin. """
     # Services that are not actually openstack projects but are used by them
     OST_SERVICES_DEPS = ['dnsmasq',
                          r'(?:nfs-)?ganesha\S*',
                          'haproxy',
-                         r"keepalived{}".format(OSTProject.SVC_VALID_SUFFIX),
-                         r"vault{}".format(OSTProject.SVC_VALID_SUFFIX),
+                         rf"keepalived{OSTProject.SVC_VALID_SUFFIX}",
+                         rf"vault{OSTProject.SVC_VALID_SUFFIX}",
                          r'qemu-system-\S+',
                          'radvd',
                          ]
@@ -484,7 +553,7 @@ class OSTProjectCatalog(object):
         if name in self._projects:
             return self._projects[name]
 
-        raise AttributeError("project {} not found".format(name))
+        raise AttributeError(f"project {name} not found")
 
     @cached_property
     def all(self):
@@ -514,7 +583,8 @@ class OSTProjectCatalog(object):
         return sorted(masked)
 
     def add(self, name, *args, **kwargs):
-        self._projects[name] = OSTProject(name, *args, **kwargs)
+        params = OSTProjectParameters(name=name, *args, **kwargs)
+        self._projects[name] = OSTProject(params)
 
     @cached_property
     def packages_core_exprs(self):
@@ -533,8 +603,8 @@ class OSTProjectCatalog(object):
         return list(deps)
 
 
-class OSTServiceBase(object):
-
+class OSTServiceBase():
+    """ Representation of an Openstack service. """
     def __init__(self, name, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.nethelp = host_helpers.HostNetworkingHelper()
