@@ -1,6 +1,7 @@
-from .common import OpenvSwitchChecksBase  # noqa: F403,F401
+from .common import OpenvSwitchChecks  # noqa: F403,F401
 from .ovs import (  # noqa: F403,F401
     OpenvSwitchBase,
+    OVSBFD,
     OVSDB,
     OVSDPLookups,
     OVSBridge,
