@@ -1,5 +1,6 @@
 import abc
 from functools import cached_property
+from dataclasses import dataclass
 
 from hotsos.core.log import log
 from hotsos.core.search import (
@@ -14,15 +15,16 @@ from hotsos.core.host_helpers import SystemdHelper
 from hotsos.core.plugins.openvswitch.common import OVS_SERVICES_EXPRS
 
 
-class OVNSBDBPort(object):
+@dataclass
+class OVNSBDBPort:
+    """ Representation of OVS SBDB port """
 
-    def __init__(self, name, port_type):
-        self.name = name
-        self.type = port_type
+    name: str
+    port_type: str
 
 
-class OVNSBDBChassis(object):
-
+class OVNSBDBChassis():
+    """ Representation of OVS SBDB chassis """
     def __init__(self, name, content):
         self.name = name
         self.content = content or {}
@@ -47,11 +49,11 @@ class OVNSBDBChassis(object):
 
     @cached_property
     def cr_ports(self):
-        return [p for p in self._ports if p.type == 'cr-lrp']
+        return [p for p in self._ports if p.port_type == 'cr-lrp']
 
 
-class OVNDBBase(object):
-
+class OVNDBBase():
+    """ Base class for OVN database objects. """
     def __init__(self):
         contents = mktemp_dump(''.join(self.db_show))
         s = FileSearcher()
@@ -140,10 +142,8 @@ class OVNSBDB(OVNDBBase):
         return _chassis
 
 
-class OVNBase(object):
-    """
-    Base class for all OVN components.
-    """
+class OVNBase():
+    """ Base class for all OVN components. """
     def __init__(self):
         self.systemd = SystemdHelper(service_exprs=OVS_SERVICES_EXPRS)
 
@@ -178,7 +178,11 @@ class OVNBase(object):
         if self.is_ovn_central:
             return OVNSBDB()
 
+        return None
+
     @cached_property
     def nbdb(self):
         if self.is_ovn_central:
             return OVNNBDB()
+
+        return None
