@@ -2,7 +2,8 @@ from hotsos.core.plugins.system.system import SystemBase
 from hotsos.core import plugintools
 
 
-class SystemChecksBase(SystemBase, plugintools.PluginPartBase):
+class SystemChecks(SystemBase, plugintools.PluginPartBase):
+    """ System checks. """
     plugin_name = 'system'
     plugin_root_index = 1
 
