@@ -8,9 +8,16 @@ from functools import cached_property
 from hotsos.core.config import HotSOSConfig
 from hotsos.core.host_helpers import CLIHelper
 from hotsos.core.log import log
+from hotsos.core.exceptions import MismatchError, InvalidPathError
 
 
-class DataRootManager(object):
+class DataRootManager():
+    """
+    Context manager to manager the "data root" used by HotSOS during its
+    execution. Data roots can either be system root / or an unpacked soseport
+    which can be provided compressed and HotSOS will take care of unpacking it
+    here.
+    """
     TYPE_HOST = 0
     TYPE_SOSREPORT = 1
 
@@ -39,8 +46,8 @@ class DataRootManager(object):
 
         try:
             shutil.rmtree(self.tmpdir)
-        except Exception:
-            os.system('chmod -R 777 {}'.format(self.tmpdir))
+        except PermissionError:
+            os.system(f'chmod -R 777 {self.tmpdir}')
             shutil.rmtree(self.tmpdir)
 
     @cached_property
@@ -71,11 +78,13 @@ class DataRootManager(object):
                 rootdir = tar.firstmember.name.partition('/')[0]
                 target = os.path.join(self.tmpdir, rootdir)
                 if not os.path.exists(target):
-                    sys.stdout.write("INFO: extracting sosreport {} to {}\n".
-                                     format(path, target))
+                    sys.stdout.write(f"INFO: extracting sosreport {path} to "
+                                     "{target}\n")
                     try:
                         tar.extractall(self.tmpdir)
-                    except Exception:
+                    # We really do want to catch all here since we don't care
+                    # why it failed but don't want to fail hard if it does.
+                    except Exception:  # pylint: disable=W0718
                         log.exception("error occured while unpacking "
                                       "sosreport:")
                         # some members might fail to extract e.g. permission
@@ -88,12 +97,12 @@ class DataRootManager(object):
                                          "be incomplete as a result)\n")
                         tar.extractall(self.tmpdir)
                 else:
-                    sys.stdout.write("INFO: target {} already exists - "
-                                     "skipping unpack\n".format(target))
+                    sys.stdout.write(f"INFO: target {target} already exists - "
+                                     "skipping unpack\n")
 
             return target
 
-        raise Exception("invalid data root '{}'".format(path))
+        raise InvalidPathError(f"invalid data root '{path}'")
 
     def _type(self, path):
         if path == '/':
@@ -111,15 +120,15 @@ class DataRootManager(object):
         if self.type == self.TYPE_HOST:
             return 'localhost'
 
-        return 'sosreport {}'.format(self.data_root)
+        return f'sosreport {self.data_root}'
 
     @property
     def basename(self):
         path = self.data_root
         if path == '/':
             if HotSOSConfig.data_root != self.data_root:
-                raise Exception("HotSOSConfig.data_root != {}".
-                                format(self.data_root))
+                raise MismatchError(
+                    f"HotSOSConfig.data_root != {self.data_root}")
 
             return CLIHelper().hostname()
 
