@@ -10,16 +10,26 @@ from propertree.propertree2 import (
 )
 from hotsos.core.log import log
 from hotsos.core.host_helpers.config import ConfigBase
+from hotsos.core.exceptions import (
+    NotEnoughParametersError,
+    MissingRequiredParameterError,
+    UnexpectedParameterError
+)
 
 
 class ImportPathIsNotAClass(Exception):
+    """ Exception used when an import is not a class. """
     def __init__(self, msg):
         self.msg = msg
         super().__init__()
 
 
 class YDefsSection(PTreeSection):
-
+    """
+    Implementation of PTreeSection that ensures a context is always provided.
+    This is used by all YAML handler types to load their defintitions e.g. of
+    events or scenarios.
+    """
     def __init__(self, name, content, *args, context=None, **kwargs):
         """
         @param name: name of defs group
@@ -43,7 +53,7 @@ class YDefsContext(UserDict):
         """
         @param initial_state: optional dict to use as initial state.
         """
-        self.data = initial_state or {}
+        super().__init__(initial_state or {})
 
     def __setitem__(self, key, item):
         # don't print values as they can be very large and e.g. search
@@ -56,7 +66,7 @@ class YDefsContext(UserDict):
         return self.data.get(key)
 
 
-class PropertyCacheRefResolver(object):
+class PropertyCacheRefResolver():
     """
     This class is used to resolve string references to property cache entries.
     """
@@ -64,16 +74,16 @@ class PropertyCacheRefResolver(object):
         log.debug("%s: resolving '%s'", self.__class__.__name__, refstr)
         self.refstr = refstr
         if not self.is_valid_cache_ref(refstr):
-            msg = ("{} is not a valid property cache reference or variable".
-                   format(refstr))
-            raise Exception(msg)
+            msg = (f"{refstr} is not a valid property cache reference or "
+                   "variable")
+            raise NameError(msg)
 
         self.vars = pcrr_vars
         self.checks = checks
         if self.reftype == 'checks' and checks is None:
-            msg = ("{} is a checks cache reference but checks dict not "
-                   "provided".format(refstr))
-            raise Exception(msg)
+            msg = (f"{refstr} is a checks cache reference but checks dict not "
+                   "provided")
+            raise NotEnoughParametersError(msg)
 
     @property
     def reftype(self):
@@ -91,7 +101,7 @@ class PropertyCacheRefResolver(object):
             # This is an implementation of YPropertyChecks
             return "checks"
 
-        raise Exception("unknown ref type")
+        raise NameError(f"unknown ref type {self.refstr}")
 
     @property
     def _ref_body(self):
@@ -101,7 +111,7 @@ class PropertyCacheRefResolver(object):
         if self.reftype == 'variable':
             prefix = "$"
         elif self.reftype == 'checks':
-            prefix = "@checks.{}.".format(self.check_name)
+            prefix = f"@checks.{self.check_name}."
 
         return self.refstr.partition(prefix)[2]
 
@@ -124,14 +134,17 @@ class PropertyCacheRefResolver(object):
     @property
     def check_name(self):
         if self.reftype != 'checks':
-            raise Exception("ref does not have type 'checks'")
+            raise MissingRequiredParameterError(
+                "ref does not have type 'checks'")
 
         return self.refstr.partition('@checks.')[2].partition('.')[0]
 
     @property
     def property_name(self):
-        if self.reftype == 'checks':
-            return self._ref_body.partition('.')[0]
+        if self.reftype != 'checks':
+            return None
+
+        return self._ref_body.partition('.')[0]
 
     @property
     def property_cache_key(self):
@@ -139,7 +152,8 @@ class PropertyCacheRefResolver(object):
         if self.reftype == 'checks':
             _key = self._ref_body.partition('.')[2]
         else:
-            raise Exception("not a check reftype")
+            raise UnexpectedParameterError(
+                f"not a check reftype: {self.reftype}")
 
         # strip func if exists
         return _key.partition(':')[0]
@@ -171,7 +185,7 @@ class PropertyCacheRefResolver(object):
         extras = {'comma_join': {
                     'requirements': [lambda value: isinstance(value, list),
                                      lambda value: isinstance(value, dict)],
-                    'action': lambda value: ', '.join(value)},
+                    'action': ', '.join},
                   'unique_comma_join': {
                     'requirements': [lambda value: isinstance(value, list),
                                      lambda value: isinstance(value, dict)],
@@ -199,7 +213,7 @@ class PropertyCacheRefResolver(object):
         """
         val = []
         group = int(cache_key.partition('results_group_')[2])
-        for result in self.checks[self.check_name]._search_results:
+        for result in self.checks[self.check_name].search_results:
             if len(result) >= group:
                 val.append(result.get(group))
 
@@ -227,7 +241,7 @@ class PropertyCacheRefResolver(object):
             val = self.vars.resolve(varname)
 
         if val is None:
-            return
+            return None
 
         func = self.property_cache_value_renderer_function
         if not func:
@@ -238,7 +252,12 @@ class PropertyCacheRefResolver(object):
 
 
 class PropertyCache(UserDict):
-
+    """
+    A cached used by YAML property objects. This is useful where we want to
+    retain state for the lifetime of a property object instance. Note that this
+    state does and will not persist a reload of the YAML tree i.e. if the
+    property objects are recreated the caches are as well.
+    """
     def merge(self, cache):
         if not isinstance(cache, self.__class__):
             log.error("attempt to merge cache failed - provided cache is not "
@@ -262,12 +281,15 @@ class PropertyCache(UserDict):
     def __getattr__(self, key):
         log.debug("%s: fetching key=%s (exists=%s)", self.id, key,
                   key in self.data)
-        if key in self.data:
-            return self.data[key]
+        return self.data.get(key)
 
 
 class YPropertyBase(PTreeOverrideBase):
-
+    """
+    Base class used for all YAML property objects. Implements and extends
+    PTreeOverrideBase to provide frequently used methods and helpers to
+    property implementations.
+    """
     def __init__(self, *args, **kwargs):
         self._cache = PropertyCache()
         super().__init__(*args, **kwargs)
@@ -285,6 +307,9 @@ class YPropertyBase(PTreeOverrideBase):
                 _name = name.partition('$')[2]
                 return self.context.vars.resolve(_name)
 
+        log.warning("could not resolve var '%s' - vars not found in "
+                    "context", name)
+
         return name
 
     @property
@@ -302,20 +327,24 @@ class YPropertyBase(PTreeOverrideBase):
         """
         if self.context is None:
             log.info("context not available - cannot load '%s'", key)
-            return
+            return None
 
         # we save all imports in a dict called "import_cache" within the
         # global context so that all properties have access.
         c = getattr(self.context, 'import_cache')
-        if c:
-            return c.get(key)
+        if not c:
+            return None
 
-    def _get_mod_class_from_path(self, path):
+        return c.get(key)
+
+    @staticmethod
+    def _get_mod_class_from_path(path):
         _mod = path.rpartition('.')[0]
         _cls = path.rpartition('.')[2]
         return _mod, _cls
 
-    def _get_class_property_from_path(self, path, no_property=False):
+    @staticmethod
+    def _get_class_property_from_path(path, no_property=False):
         log.debug("fetching class and property from path (no_property=%s)",
                   no_property)
         # first strip any factory class info and add back to prop at end.
@@ -420,10 +449,14 @@ class YPropertyBase(PTreeOverrideBase):
             log.exception("class '%s' import failed", _cls)
             raise
 
-        key = "{}.object".format(cls)
+        key = f"{cls}.object"
         cls_inst = self._load_from_import_cache(key)
         if not cls_inst:
-            cls_inst = cls()
+            try:
+                cls_inst = cls(global_searcher=self.context.global_searcher)
+            except TypeError:
+                cls_inst = cls()
+
             self._add_to_import_cache(key, cls_inst)
 
         if ':' in _prop:
@@ -449,7 +482,8 @@ class YPropertyBase(PTreeOverrideBase):
         self._add_to_import_cache(import_str, _obj)
         return _obj
 
-    def get_method(self, import_str):
+    @staticmethod
+    def get_method(import_str):
         """ Import and instantiate Python class then call method.
 
         @param import_str: import path to Python class with method name after
@@ -469,7 +503,8 @@ class YPropertyBase(PTreeOverrideBase):
 
         return ret
 
-    def get_attribute(self, import_str):
+    @staticmethod
+    def get_attribute(import_str):
         log.debug("fetching attribute %s", import_str)
         mod = import_str.rpartition('.')[0]
         attr = import_str.rpartition('.')[2]
@@ -496,15 +531,13 @@ class YPropertyBase(PTreeOverrideBase):
             return self.get_property(import_str)
         except TypeError as exc:
             log.debug("get_property failed, trying get_attribute: %s", exc)
-        except Exception:
-            log.exception("get_property failed for unknown reason")
 
         return self.get_attribute(import_str)
 
 
 class YPropertyOverrideBase(YPropertyBase, PTreeOverrideBase):
-    pass
+    """ Base class for all simple/flat property objects. """
 
 
 class YPropertyMappedOverrideBase(YPropertyBase, PTreeMappedOverrideBase):
-    pass
+    """ Base class for all mapped property objects. """
