@@ -9,8 +9,8 @@ from hotsos.core.ycheck.engine.properties.common import (
 )
 
 
-class YPropertyInputBase(object):
-
+class YPropertyInputBase():
+    """ Base class for input property implementations. """
     @property
     def options(self):
         defaults = {'disable-all-logs': False,
@@ -18,18 +18,18 @@ class YPropertyInputBase(object):
                     'kwargs': {},
                     'args-callback': None}
 
-        if isinstance(self.content, dict):
-            _options = self.content.get('options', defaults)
+        if isinstance(self.content, dict):  # pylint: disable=E1101
+            _options = self.content.get('options', defaults)  # noqa, pylint: disable=E1101
             defaults.update(_options)
 
         return defaults
 
     @property
     def command(self):
-        if not isinstance(self.content, dict):
-            return
+        if not isinstance(self.content, dict):  # pylint: disable=E1101
+            return None
 
-        return self.content.get('command')
+        return self.content.get('command')  # pylint: disable=E1101
 
     def expand_paths(self, paths):
         _paths = []
@@ -37,7 +37,7 @@ class YPropertyInputBase(object):
             path = os.path.join(HotSOSConfig.data_root, path)
             if (HotSOSConfig.use_all_logs and not
                     self.options['disable-all-logs']):
-                path = "{}*".format(path)
+                path = f"{path}*"
 
             _paths.append(path)
 
@@ -45,16 +45,16 @@ class YPropertyInputBase(object):
 
     @property
     def path(self):
-        raise Exception("do not call this directly")
+        raise AttributeError("do not call this directly")
 
     @cached_property
     def paths(self):
         _paths = []
         fs_path = None
-        if isinstance(self.content, (str, list)):
-            fs_path = self.content
+        if isinstance(self.content, (str, list)):  # pylint: disable=E1101
+            fs_path = self.content  # pylint: disable=E1101
         else:
-            fs_path = self.content.get('path')
+            fs_path = self.content.get('path')  # pylint: disable=E1101
 
         if fs_path:
             if isinstance(fs_path, list):
@@ -66,7 +66,7 @@ class YPropertyInputBase(object):
             return self.expand_paths(_paths)
 
         if self.command:
-            cmd_tmp_path = self.cache.cmd_tmp_path
+            cmd_tmp_path = self.cache.cmd_tmp_path  # pylint: disable=E1101
             if cmd_tmp_path:
                 return [cmd_tmp_path]
 
@@ -79,15 +79,22 @@ class YPropertyInputBase(object):
 
             with CLIHelperFile(delete_temp=False) as cli:
                 outfile = getattr(cli, self.command)(*args, **kwargs)
-                self.cache.set('cmd_tmp_path', outfile)
+                self.cache.set('cmd_tmp_path', outfile)  # noqa, pylint: disable=E1101
                 return [outfile]
 
         log.debug("no input provided")
+        return None
 
 
 class YPropertyInput(YPropertyOverrideBase, YPropertyInputBase):
-    _override_keys = ['input']
+    """
+    Input property. Provides support for defining input such a file or command
+    output.
+
+    This is typically used in conjunction with the search property.
+    """
+    override_keys = ['input']
     # We want to be able to use this property both on its own and as a member
     # of other mapping properties e.g. Checks. The following setting enables
     # this.
-    _override_auto_implicit_member = False
+    override_auto_implicit_member = False
