@@ -6,9 +6,12 @@ from hotsos.core.ycheck.engine.properties.requires import (
 
 
 class YRequirementTypeProperty(YRequirementTypeWithOpsBase):
-    """ Provides logic to perform checks on Python Properties. """
-    _override_keys = ['property']
-    _overrride_autoregister = True
+    """
+    Property requires type property. Provides support for defining checks on
+    python properties.
+    """
+    override_keys = ['property']
+    override_autoregister = True
 
     @property
     @intercept_exception
