@@ -3,10 +3,20 @@ from hotsos.core.ycheck.engine.properties.common import YPropertyOverrideBase
 
 
 class YPropertyVars(YPropertyOverrideBase):
-    _override_keys = ['vars']
+    """
+    Vars property. Provides support for defining variables in YAML e.g.
+
+    vars:
+      var1: val1
+      var2: @property1
+
+    where val1 is a literal and property2 is an import path.
+    """
+    override_keys = ['vars']
     _allow_subtree = False
 
-    def _is_import_path(self, value):
+    @staticmethod
+    def _is_import_path(value):
         return isinstance(value, str) and value.startswith('@')
 
     def resolve(self, name):
@@ -18,8 +28,8 @@ class YPropertyVars(YPropertyOverrideBase):
         """
         log.debug("resolving var '%s'", name)
         if not name:
-            raise Exception("attempting to resolve invalid varname '{}'".
-                            format(name))
+            raise NameError(
+                f"attempting to resolve invalid varname '{name}'")
 
         if hasattr(self, name):
             value = getattr(self, name)
