@@ -1,7 +1,7 @@
 import re
 
 from hotsos.core.host_helpers import CLIHelper, HostNetworkingHelper
-from hotsos.core.plugins.openstack.common import OpenstackChecksBase
+from hotsos.core.plugins.openstack.common import OpenstackBase, OpenStackChecks
 from hotsos.core.plugins.openstack.neutron import (
     IP_HEADER_BYTES,
     GRE_HEADER_BYTES,
@@ -12,20 +12,26 @@ from hotsos.core.issues import (
     IssuesManager,
     OpenstackWarning,
 )
+from hotsos.core.plugintools import (
+    summary_entry,
+    get_min_available_entry_index,
+)
 
 
-class OpenstackNetworkChecks(OpenstackChecksBase):
+class OpenstackNetworkChecks(OpenstackBase, OpenStackChecks):
+    """ Implements OpenStack network checks. """
     summary_part_index = 4
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
         self.cli = CLIHelper()
 
     @property
     def summary_subkey(self):
         return 'network'
 
-    def _get_port_stat_outliers(self, counters):
+    @staticmethod
+    def _get_port_stat_outliers(counters):
         """ For a given port's packet counters, identify outliers i.e. > 1%
         and create a new dict with count and percent values.
         """
@@ -44,7 +50,7 @@ class OpenstackNetworkChecks(OpenstackChecksBase):
                     if rxtx not in stats:
                         stats[rxtx] = {}
 
-                    stats[rxtx][key] = "{} ({}%)".format(int(value), pcent)
+                    stats[rxtx][key] = f"{int(value)} ({pcent}%)"
 
         return stats
 
@@ -80,17 +86,24 @@ class OpenstackNetworkChecks(OpenstackChecksBase):
 
         return port_health_info
 
-    def __9_summary_config(self):
+    @summary_entry('config', get_min_available_entry_index() + 5)
+    def summary_config(self):
         config_info = self.get_config_info()
         if config_info:
             return config_info
 
-    def __10_summary_phy_port_health(self):
+        return None
+
+    @summary_entry('phy-port-health', get_min_available_entry_index() + 6)
+    def summary_phy_port_health(self):
         port_health_info = self.get_phy_port_health_info()
         if port_health_info:
             return port_health_info
 
-    def __11_summary_namespaces(self):
+        return None
+
+    @summary_entry('namespaces', get_min_available_entry_index() + 7)
+    def summary_namespaces(self):
         """Populate namespace information dict."""
         ns_info = {}
         for line in self.cli.ip_netns():
@@ -104,66 +117,79 @@ class OpenstackNetworkChecks(OpenstackChecksBase):
         if ns_info:
             return ns_info
 
-    def __12_summary_router_port_mtus(self):
-        """ Provide a summary of ml2-ovs router port mtus. """
-        phy_mtus = set()
+        return None
+
+    def _get_router_iface_mtus(self):
+        """
+        Get the mtu of each qr and qg port in every qrouter or snat namespace.
+
+        @return: dictionary of port prefixes and list of mtus associated with
+                 those prefixes.
+        """
         router_mtus = {}
+        for ns in self.cli.ip_netns():
+            # strip index
+            ns = ns.partition(" ")[0]
+            for nsprefix in ['qrouter', 'snat']:
+                if not ns.startswith(nsprefix):
+                    continue
+
+                for port in HostNetworkingHelper().get_ns_interfaces(ns):
+                    for pprefix in ['qr', 'sg']:
+                        if not port.name.startswith(pprefix):
+                            continue
+
+                        if pprefix not in router_mtus:
+                            router_mtus[pprefix] = set()
+
+                        router_mtus[pprefix].add(port.mtu)
+
+        return {prefix: list(mtus) for prefix, mtus in router_mtus.items()}
+
+    @summary_entry('router-port-mtus', get_min_available_entry_index() + 8)
+    def summary_router_port_mtus(self):
+        """ Provide a summary of ml2-ovs router port mtus. """
         project = getattr(self, 'neutron')
-        if project and project.bind_interfaces:
-            for port in project.bind_interfaces.values():
-                phy_mtus.add(port.mtu)
+        if not (project and project.bind_interfaces):
+            return None
 
-            for ns in self.cli.ip_netns():
-                # strip index
-                ns = ns.partition(" ")[0]
-                for nsprefix in ['qrouter', 'snat']:
-                    if not ns.startswith(nsprefix):
-                        continue
+        phy_mtus = set()
+        for port in project.bind_interfaces.values():
+            phy_mtus.add(port.mtu)
 
-                    for port in HostNetworkingHelper().get_ns_interfaces(ns):
-                        for pprefix in ['qr', 'sg']:
-                            if not port.name.startswith(pprefix):
-                                continue
+        tunnels = OpenvSwitchBase().tunnels
+        # ip header
+        overhead = IP_HEADER_BYTES
+        if 'vxlan' in tunnels:
+            overhead += VXLAN_HEADER_BYTES
+        else:
+            # gre
+            overhead += GRE_HEADER_BYTES
 
-                            if pprefix not in router_mtus:
-                                router_mtus[pprefix] = set()
+        router_mtus = self._get_router_iface_mtus()
+        all_router_mtus = set()
+        for mtus in router_mtus.values():
+            all_router_mtus.update(mtus)
 
-                            router_mtus[pprefix].add(port.mtu)
-
-            tunnels = OpenvSwitchBase().tunnels
-            # ip header
-            overhead = IP_HEADER_BYTES
-            if 'vxlan' in tunnels:
-                overhead += VXLAN_HEADER_BYTES
-            else:
-                # gre
-                overhead += GRE_HEADER_BYTES
-
-            all_router_mtus = set()
-            for prefix, mtus in router_mtus.items():
-                router_mtus[prefix] = list(mtus)
-                for mtu in mtus:
-                    all_router_mtus.add(mtu)
-
-            if phy_mtus and all_router_mtus:
-                smallest_allowed = min(phy_mtus) - overhead
-                if max(all_router_mtus) > smallest_allowed:
-                    msg = ("This Neutron L3 agent host has one or more router "
-                           "ports with mtu={} which is greater or equal to "
-                           "the smallest allowed ({}) on "
-                           "the physical network. This will result in dropped "
-                           "packets or unexpected fragmentation in overlay "
-                           "networks.".format(max(all_router_mtus),
-                                              smallest_allowed))
-                    IssuesManager().add(OpenstackWarning(msg))
+        if phy_mtus and all_router_mtus:
+            smallest_allowed = min(phy_mtus) - overhead
+            if max(all_router_mtus) > smallest_allowed:
+                msg = ("This Neutron L3 agent host has one or more router "
+                       f"ports with mtu={max(all_router_mtus)} which is "
+                       "greater or equal to the smallest allowed "
+                       f"({smallest_allowed}) on the physical network. "
+                       "This will result in dropped packets or "
+                       "unexpected fragmentation in overlay networks.")
+                IssuesManager().add(OpenstackWarning(msg))
 
         return router_mtus
 
-    def __13_summary_vm_port_health(self):
+    @summary_entry('vm-port-health', get_min_available_entry_index() + 9)
+    def summary_vm_port_health(self):
         """ For each instance get its ports and check port health, reporting on
         any outliers. """
         if not self.nova.instances:
-            return
+            return None
 
         port_health_info = {}
         for guest in self.nova.instances.values():
@@ -183,3 +209,5 @@ class OpenstackNetworkChecks(OpenstackChecksBase):
             health = {'num-vms-checked': len(self.nova.instances),
                       'stats': port_health_info}
             return health
+
+        return None
